package v1

import (
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"os"
	"strings"
	"time"
	"xingjia-management-gateway/errors"
	"xingjia-management-gateway/impl/v1/local_objs"
	param_v1 "xingjia-management-gateway/param/v1"
	"xingjia-management-gateway/parser"
	"xingjia-management-gateway/utils"
)

// UploadFile godoc
// @Summary 上传文件
// @Description 上传文件
// @Tags upload
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param file formData file false "file"
// @Success 200 {object} v1.UploadResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/file [post]
func (c Controller) Upload(ctx *gin.Context) {
	// 解析参数
	parseParamTask := func() error {
		file, err := ctx.FormFile("file")
		if err != nil {
			fmt.Printf("upload get file:%v\n", err)
			logger.Error("func",
				zap.String("call", "ctx.FormFile"),
				zap.String("error", err.Error()))
			return errors.SystemError
		}

		array := strings.Split(file.Filename, ".")
		tail := ""
		if len(array) > 0 {
			tail = array[len(array)-1]
		}
		newName := fmt.Sprintf("%d-%s.%s", time.Now().Unix(), utils.GenerateRandomStr(6, "num"), tail)
		err = ctx.SaveUploadedFile(file, parser.Conf.ObjPath+newName)
		if err != nil {
			logger.Error("func",
				zap.String("call", "SaveUploadedFile"),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		err = local_objs.AddToDb(database.DB(), newName)
		if err != nil {
			os.Remove(parser.Conf.ObjPath + newName)
			return err
		}
		retUrl := parser.Conf.ObjUrl + "?" + "obj=" + newName

		// 响应数据
		resp := &param_v1.UploadResponse{}
		resp.Data = retUrl

		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	httptasker.Exec(ctx, parseParamTask)
}

//
// @Summary 获取文件
// @Description 获取文件
// @Tags download
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param obj query string true " "
// @Success 200 {object} v1.UploadResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/file [get]
func (c Controller) DownLoad(ctx *gin.Context) {
	// 解析参数
	req := param_v1.DownLoadRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.DownLoadQuery, nil)
		if err != nil {
			fmt.Printf("upload get file:%v\n", err)
			logger.Error("func",
				zap.String("call", "ctx.FormFile"),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		ctx.File(parser.Conf.ObjPath + req.Obj)
		return nil
	}

	httptasker.Exec(ctx, parseParamTask)
}
