// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package contact

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"time"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/consts"
	"xingjia-management-gateway/errors"
	"xingjia-management-gateway/impl/v1/operation_log"
	dbmodel "xingjia-management-gateway/model"
)

func checkContactDelParam(req *apis.ContactDelRequest) error {
	switch {
	case req.Id == 0:
		return status.Error(10003, "删除目标不能为空")
	case req.SelfId == 0:
		return status.Error(10003, "未知操作者")
	}
	return nil
}

//
func ContactDel(ctx context.Context, req *apis.ContactDelRequest) (reply *apis.ContactDelReply, err error) {

	reply = &apis.ContactDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkContactDelParam(req); err != nil {
		return nil, err
	}
	now := time.Now()
	old := dbmodel.TContact{}

	where := map[string]interface{}{
		"id": req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()

	p := dbmodel.TContact{}
	err = p.Delete(db, where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	// 操作日志
	logReq := &apis.OperationLogAddRequest{
		OriginContent: getContactContent(old),
		TargetContent: "",
		Uid:           req.SelfId,
		User:          req.SelfName,
		Module:        consts.OperationModuleContact,
		OperationTime: now.Unix(),
		Operation:     consts.OperationActionDel,
	}
	err = operation_log.OperationLogAddHandle(logReq, db)
	if err != nil {
		return nil, err
	}

	db.Commit()

	return reply, nil
}
