// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package contact

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/errors"
	dbmodel "xingjia-management-gateway/model"
)

//
func ContactInfo(ctx context.Context, req *apis.ContactInfoRequest) (reply *apis.ContactInfoReply, err error) {

	reply = &apis.ContactInfoReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	p := dbmodel.TContact{}
	where := map[string]interface{}{
		"id": 1,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	reply.Addr = p.Addr
	reply.Phone = p.Phone
	reply.Email = p.Email
	reply.Id = p.ID

	return reply, nil
}
