// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package jt

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"time"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/consts"
	"xingjia-management-gateway/errors"
	"xingjia-management-gateway/impl/v1/operation_log"
	dbmodel "xingjia-management-gateway/model"
)

func checkJtContentPublishParam(req *apis.JtContentPublishRequest) error {
	switch {
	case consts.JtContentTable[req.ContentType] == "":
		return status.Error(10003, "未知内容类型")
	case req.SelfId == 0:
		return status.Error(10003, "未知操作者")
	case req.Id == 0:
		return status.Error(10003, "目标不能为空")
	case req.PublishStatus != 1 && req.PublishStatus != 2:
		return status.Error(10003, "请选择是否发布")
	}
	return nil
}

//
func JtContentPublish(ctx context.Context, req *apis.JtContentPublishRequest) (reply *apis.JtContentPublishReply, err error) {

	reply = &apis.JtContentPublishReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkJtContentPublishParam(req); err != nil {
		return nil, err
	}
	now := time.Now()
	old := dbmodel.NewJtContent(consts.JtContentTable[req.ContentType])

	where := map[string]interface{}{
		"id": req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	values := map[string]interface{}{
		"publish_status":req.PublishStatus,
		"updated_at": time.Now(),
	}

	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()
	p := dbmodel.NewJtContent(consts.JtContentTable[req.ContentType])
	err = p.Update(db, where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}


	// 操作日志
	content := getJtContent(*old)
	tmp := *old
	tmp.PublishStatus = req.PublishStatus
	tcontent := getJtContent(tmp)
	logReq := &apis.OperationLogAddRequest{
		OriginContent: content,
		TargetContent: tcontent,
		Uid:           req.SelfId,
		User:          req.SelfName,
		Module:        int32(req.ContentType),
		OperationTime: now.Unix(),
		Operation:     consts.OperationActionUpdate,
	}
	err = operation_log.OperationLogAddHandle(logReq, db)
	if err != nil {
		return nil, err
	}

	db.Commit()

	return reply, nil
}
