// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package jt

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"strings"
	"time"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/consts"
	"xingjia-management-gateway/errors"
	"xingjia-management-gateway/impl/v1/local_objs"
	"xingjia-management-gateway/impl/v1/operation_log"
	dbmodel "xingjia-management-gateway/model"
)

func checkJtContentUpdateParam(req *apis.JtContentUpdateRequest) error {
	switch {
	case consts.JtContentTable[req.ContentType] == "":
		return status.Error(10003, "未知内容类型")
	case req.SelfId == 0:
		return status.Error(10003, "未知操作者")
	case req.Id == 0:
		return status.Error(10003, "目标不能为空")
	case req.PublishStatus != 1 && req.PublishStatus != 2:
		return status.Error(10003, "请选择是否发布")
	}
	return nil
}

//
func JtContentUpdate(ctx context.Context, req *apis.JtContentUpdateRequest) (reply *apis.JtContentUpdateReply, err error) {

	reply = &apis.JtContentUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkJtContentUpdateParam(req); err != nil {
		return nil, err
	}
	now := time.Now()
	old := dbmodel.NewJtContent(consts.JtContentTable[req.ContentType])

	where := map[string]interface{}{
		"id": req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	values := map[string]interface{}{
		"first_pics":     strings.Join(req.FirstPics, ";"),
		"content":        req.Content,
		"publish_status": req.PublishStatus,
		"updated_at":     time.Now(),
		"title":          req.Title,
	}

	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()
	p := dbmodel.NewJtContent(consts.JtContentTable[req.ContentType])
	err = p.Update(db, where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	p.Content = values["content"].(string)
	p.FirstPics = values["first_pics"].(string)

	// 操作日志
	content := getJtContent(*old)
	tcontent := getJtContent(*p)
	logReq := &apis.OperationLogAddRequest{
		OriginContent: content,
		TargetContent: tcontent,
		Uid:           req.SelfId,
		User:          req.SelfName,
		Module:        int32(req.ContentType),
		OperationTime: now.Unix(),
		Operation:     consts.OperationActionUpdate,
	}
	err = operation_log.OperationLogAddHandle(logReq, db)
	if err != nil {
		return nil, err
	}
	inList := req.FirstPics
	array := local_objs.GetObjFromText(req.Content)
	inList = append(inList, array...)

	outList := strings.Split(old.FirstPics, ";")
	array2 := local_objs.GetObjFromText(old.Content)
	outList = append(outList, array2...)
	err = local_objs.InAndOut(db, inList, outList)
	if err != nil {
		return nil, err
	}

	db.Commit()

	return reply, nil
}
