// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package operation_log

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/errors"
	dbmodel "xingjia-management-gateway/model"
)

func checkOperationLogDelParam(req *apis.OperationLogDelRequest) error {
	switch {
	case req.SelfId == 0:
		return status.Error(10003, "未知操作者")
	case len(req.Ids) == 0:
		return status.Error(10003, "目标不能为空")
	}
	return nil
}

//
func OperationLogDel(ctx context.Context, req *apis.OperationLogDelRequest) (reply *apis.OperationLogDelReply, err error) {

	reply = &apis.OperationLogDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkOperationLogDelParam(req); err != nil {
		return nil, err
	}
	if err = checkAdmin(req.SelfId); err != nil {
		return nil, err
	}
	where := map[string]interface{}{
		"id in": req.Ids,
	}

	p := &dbmodel.TOperationLog{}
	err = p.Delete(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	return reply, err
}
