// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package operation_log

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/consts"
	"xingjia-management-gateway/errors"
	dbmodel "xingjia-management-gateway/model"
)

func checkOperationLogListParam(req *apis.OperationLogListRequest) error {
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	return nil
}

//
func OperationLogList(ctx context.Context, req *apis.OperationLogListRequest) (reply *apis.OperationLogListReply, err error) {

	reply = &apis.OperationLogListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkOperationLogListParam(req); err != nil {
		return nil, err
	}
	where := map[string]interface{}{}
	if req.Start > 0 {
		where["operation_time >="] = req.Start
	}
	if req.End > 0 {
		where["operation_time <="] = req.End
	}
	p := &dbmodel.TOperationLog{}
	reply.Page = req.Page

	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}

	reply.List = make([]*apis.OperationLogItem, len(list))
	for i, v := range list {
		item := &apis.OperationLogItem{
			Id:            v.ID,
			User:          v.User,
			Uid:           v.Uid,
			OriginContent: v.OriginContent,
			TargetContent: v.TargetContent,
			OperationTime: v.OperationTime,
			Operation: consts.OperationActionName[int(v.Operation)],
			Module: consts.OperationModuleName[int(v.Module)],
		}
		reply.List[i] = item
	}

	return reply, err
}
