// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package page_pic

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"time"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/consts"
	"xingjia-management-gateway/errors"
	"xingjia-management-gateway/impl/v1/local_objs"
	"xingjia-management-gateway/impl/v1/operation_log"
	dbmodel "xingjia-management-gateway/model"
)

func checkPagePicAddParam(req *apis.PagePicAddRequest) error {
	switch {
	case req.SelfId == 0:
		return status.Error(10003, "未知操作者")
	case req.Pic == "":
		return status.Error(10003, "图片不能为空")
	}
	return nil
}

func getPagePic(p dbmodel.TPagePic) string {
	m := map[string]interface{}{
		"链接": p.Pic,
	}
	bytes, _ := json.Marshal(m)
	return string(bytes)
}

//
func PagePicAdd(ctx context.Context, req *apis.PagePicAddRequest) (reply *apis.PagePicAddReply, err error) {

	reply = &apis.PagePicAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkPagePicAddParam(req); err != nil {
		return nil, err
	}
	now := time.Now()
	p := dbmodel.TPagePic{}
	p.Pic = req.Pic
	p.CreatedAt = now
	p.UpdatedAt = now

	db := database.DB().Begin()
	err = p.Insert(db)
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()
	if err != nil {
		return nil, errors.DataBaseError
	}

	// 操作日志
	content := getPagePic(p)
	logReq := &apis.OperationLogAddRequest{
		OriginContent: "",
		TargetContent: content,
		Uid:           req.SelfId,
		User:          req.SelfName,
		Module:        consts.OperationModulePagePic,
		OperationTime: now.Unix(),
		Operation:     consts.OperationActionAdd,
	}
	err = operation_log.OperationLogAddHandle(logReq, db)
	if err != nil {
		return nil, err
	}

	inList := []string{req.Pic}
	err = local_objs.InAndOut(db, inList, []string{})
	if err != nil {
		return nil, err
	}

	db.Commit()

	return reply, nil
}
