// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package page_pic

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/errors"
	dbmodel "xingjia-management-gateway/model"
)

func checkPagePicListParam(req *apis.PagePicListRequest) error {
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	return nil
}

//
func PagePicList(ctx context.Context, req *apis.PagePicListRequest) (reply *apis.PagePicListReply, err error) {

	reply = &apis.PagePicListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkPagePicListParam(req); err != nil {
		return nil, err
	}

	p := dbmodel.TPagePic{}

	reply.Page = req.Page
	reply.Total, err = p.Count(database.DB(), nil, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), nil, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*apis.PagePicItem, len(list))
	for i, v := range list {
		item := &apis.PagePicItem{
			Id:        v.ID,
			CreatedAt: v.UpdatedAt.Unix(),
			Pic:       v.Pic,
		}
		reply.List[i] = item
	}

	return reply, nil
}
