// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"time"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/consts"
	"xingjia-management-gateway/errors"
	"xingjia-management-gateway/impl/v1/operation_log"
	dbmodel "xingjia-management-gateway/model"
)

func checkUserDelParam(req *apis.UserDelRequest) error {
	switch {
	case req.DelId == 0:
		return status.Error(10003, "删除账号不能为空")
	case req.SelfId == 0:
		return status.Error(10003, "未知操作者")
	}
	return nil
}

//
func UserDel(ctx context.Context, req *apis.UserDelRequest) (reply *apis.UserDelReply, err error) {

	reply = &apis.UserDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkUserDelParam(req); err != nil {
		return nil, err
	}
	if err = checkAdmin(req.SelfId); err != nil {
		return nil, err
	}
	if req.DelId == req.SelfId {
		return nil, status.Error(10003, "超级管理员不能删除")
	}

	old := &dbmodel.TUser{}
	where := map[string]interface{}{
		"id": req.DelId,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.RealName = old.RealName
	reply.User = old.User
	reply.UserType = old.UserType

	p := &dbmodel.TUser{}
	where = map[string]interface{}{
		"id": req.DelId,
	}
	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()

	err = p.Delete(db, where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	// 操作日志
	content := getUserContent(*old)
	logReq := &apis.OperationLogAddRequest{
		OriginContent: content,
		TargetContent: "",
		Uid:           req.SelfId,
		User:          req.SelfName,
		Module:        consts.OperationModuleAccount,
		OperationTime: time.Now().Unix(),
		Operation:     consts.OperationActionUserDel,
	}
	err = operation_log.OperationLogAddHandle(logReq, db)
	if err != nil {
		return nil, err
	}
	db.Commit()

	return reply, nil
}
