// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/errors"
	dbmodel "xingjia-management-gateway/model"
)

func checkUserInfoParam(req *apis.UserInfoRequest) error {
	if req.Uid == 0 {
		return status.Error(10003, "未知操作者")
	}

	return nil
}

//
func UserInfo(ctx context.Context, req *apis.UserInfoRequest) (reply *apis.UserInfoReply, err error) {

	reply = &apis.UserInfoReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkUserInfoParam(req); err != nil {
		return nil, err
	}

	where := map[string]interface{}{
		"id": req.Uid,
	}

	p := &dbmodel.TUser{}

	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	reply.User = p.User
	reply.RealName = p.RealName
	reply.UserType = p.UserType
	reply.EffectiveStart = p.EffectiveStart
	reply.EffectiveEnd = p.EffectiveEnd
	reply.Uid = req.Uid

	return reply, nil
}
