// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/param/base"
)

type LoginBody struct {
	User     string `form:"user"`
	Password string `form:"password"`
}

type LoginRequest struct {
	base.Header
	LoginBody
}

type LoginData struct {
	Uid      int64  `json:"uid"`
	Token    string `json:"token"`
	User     string `json:"user"`
	UserType int32  `json:"user_type"`
}

type LoginResponse struct {
	base.Result
	Data LoginData `json:"data"`
}

type TokenRequest struct {
	base.Header
}

type TokenResponse struct {
	base.Result
	Data         string `json:"data"`
	RefreshToken string `json:"refresh_token"`
}

type UserAddBody struct {
	User           string `form:"user" json:"user"`
	Password       string `form:"password" json:"password"`
	RealName       string `form:"real_name" json:"real_name"`
	UserType       int32  `form:"user_type" json:"user_type"`
	EffectiveStart int64  `form:"effective_start" json:"effective_start"`
	EffectiveEnd   int64  `form:"effective_end" json:"effective_end"`
}

type UserAddRequest struct {
	base.Header
	UserAddBody
}

type UserAddResponse struct {
	base.Result
}

type UserListQuery struct {
	Page     int64  `form:"page" json:"page"`
	PageSize int64  `form:"page_size" json:"page_size"`
	User     string `form:"user" json:"user"`
	RealName string `form:"real_name" json:"real_name"`
}

type UserListRequest struct {
	base.Header
	UserListQuery
}

type UserListResponse struct {
	base.Result
	Data apis.UserListReply `json:"data"`
}

type UserDelQuery struct {
	Id int64 `form:"id" json:"id"`
}

type UserDelRequest struct {
	base.Header
	UserDelQuery
}

type UserDelResponse struct {
	base.Result
}

type UserResetPasswordBody struct {
	Id       int64  `form:"id" json:"id"`
	Password string `form:"password" json:"password"`
}

type UserResetPasswordRequest struct {
	base.Header
	UserResetPasswordBody
}

type UserResetPasswordResponse struct {
	base.Result
}

type UserUpdateBody struct {
	User           string `form:"user" json:"user"`
	Password       string `form:"password" json:"password"`
	RealName       string `form:"real_name" json:"real_name"`
	UserType       int32  `form:"user_type" json:"user_type"`
	EffectiveStart int64  `form:"effective_start" json:"effective_start"`
	EffectiveEnd   int64  `form:"effective_end" json:"effective_end"`
	Id             int64  `form:"id" json:"id"`
}

type UserUpdateRequest struct {
	base.Header
	UserUpdateBody
}

type UserUpdateResponse struct {
	base.Result
}
